@@ -1,8 +1,8 @@
 /* GKrellM
-|  Copyright (C) 1999-2019 Bill Wilson
+|  Copyright (C) 1999-2021 Bill Wilson
 |
-|  Author:  Bill Wilson    billw@gkrellm.net
-|  Latest versions might be found at:  http://gkrellm.net
+|  Author:  Bill Wilson
+|  Latest versions might be found at:  https://gkrellm.srcbox.net
 |
 |
 |  GKrellM is free software: you can redistribute it and/or modify it
@@ -31,38 +31,20 @@
 |  as that of the covered work.
 */
 
+#ifndef GKRELLM_PRIVATE_H
+#define GKRELLM_PRIVATE_H
+
+#ifdef HAVE_CONFIGURE_H
 #include "configure.h"
+#endif
 
 #include "../src/gkrellm-sysdeps.h"
 
-#if defined(WIN32)
-// Enable getaddrinfo on win32 if we target win xp or newer
-#if _WIN32_WINNT > 0x0500
-#define HAVE_GETADDRINFO	1
-#endif
-#endif
-
-#if defined(__linux__)
-#define HAVE_GETADDRINFO	1
-#endif
-
-#if defined(__DragonFly__)
-#define HAVE_GETADDRINFO	1
-#endif
-
 #if defined(__FreeBSD__)
 #include <sys/param.h>
-#if __FreeBSD_version >= 400000
-#define HAVE_GETADDRINFO	1
-#endif
-#endif
-
-#if defined(__OpenBSD__)
-#define HAVE_GETADDRINFO	1
 #endif
 
 #if defined(__NetBSD__)
-#define HAVE_GETADDRINFO	1
 #include <sys/param.h>
 #  if __NetBSD_Version__ <= 105010000
 #    define sa_family_t unsigned char
@@ -71,28 +53,12 @@
 
 #if defined(__solaris__)
 # include <netconfig.h>
-# if defined(NC_INET6)
-#  define HAVE_GETADDRINFO	1
-# endif
-#endif
-
-#if defined(__APPLE__)
-# ifndef socklen_t
-#  define socklen_t int
-# endif
-#define HAVE_GETADDRINFO   1
 #endif
 
 #ifndef	NI_WITHSCOPEID
 #define	NI_WITHSCOPEID	0
 #endif
 
-#if !defined(__FreeBSD__) && !defined(__linux__) && !defined(__NetBSD__) \
-    && !defined(__OpenBSD__) && !defined(__solaris__) && !defined(WIN32) \
-    && !defined(__APPLE__) && !defined(__DragonFly__)
-#define  USE_LIBGTOP
-#endif
-
 #define DEBUG_SYSDEP		0x1
 #define DEBUG_SERVER		0x2
 #define DEBUG_MAIL			0x10
@@ -107,6 +73,7 @@
 #define SENSOR_TEMPERATURE  0
 #define SENSOR_FAN          1
 #define SENSOR_VOLTAGE      2
+#define SENSOR_POWER        3
 
 #define	SENSOR_GROUP_MAINBOARD	0
 #define	SENSOR_GROUP_DISK		1
@@ -156,8 +123,6 @@ extern struct GkrellmdConfig	_GK;
 
 extern gchar	*plugin_install_log;
 
-typedef	void (*GkrellmdFunc)();
-
 extern GList	*gkrellmd_client_list,
 				*gkrellmd_plugin_enable_list,
 				*gkrellmd_plugin_config_list;
@@ -171,3 +136,5 @@ void			gkrellmd_serve_setup(GkrellmdClient *client);
 GkrellmdMonitor *gkrellmd_init_mail_monitor(void);
 
 gint		gkrellm_connect_to(gchar *, gint);
+
+#endif
\ No newline at end of file
