@@ -1,8 +1,8 @@
 /* GKrellM
-|  Copyright (C) 1999-2009 Bill Wilson
+|  Copyright (C) 1999-2021 Bill Wilson
 |
-|  Author:  Bill Wilson    billw@gkrellm.net
-|  Latest versions might be found at:  http://gkrellm.net
+|  Author:  Bill Wilson
+|  Latest versions might be found at:  https://gkrellm.srcbox.net
 |
 |
 |  GKrellM is free software: you can redistribute it and/or modify it
@@ -92,16 +92,18 @@ gkrellmd_serve_data(GkrellmdMonitor *mon, gchar *line)
 		if (mp->serve_name)
 			{
 			snprintf(buf, sizeof(buf), "<%s>\n", mp->serve_name);
+			gkrellm_debug(DEBUG_SERVER, "%s", buf);
 			mp->serve_gstring = g_string_append(mp->serve_gstring, buf);
 			mp->serve_name_sent = TRUE;
 			}
 		else
 			{
-			printf("gkrellmd: %s forgot to gkrellmd_set_serve_name()\n",
+			g_warning("gkrellmd: %s forgot to gkrellmd_set_serve_name()\n",
 					mon->name);
 			return;
 			}
 		}
+	gkrellm_debug(DEBUG_SERVER,"%s", line);
 	mp->serve_gstring = g_string_append(mp->serve_gstring, line);
 	}
 
@@ -201,7 +203,7 @@ serve_cpu_data(GkrellmdMonitor *mon, gboolean first_serve)
 	for (list = cpu_list; list; list = list->next)
 		{
 		cpu = (CpuData *) list->data;
-		sprintf(buf, "%d %lu %lu %lu %lu\n", cpu->instance,
+		snprintf(buf, sizeof(buf), "%d %lu %lu %lu %lu\n", cpu->instance,
 				cpu->user, cpu->nice, cpu->sys, cpu->idle);
 		gkrellmd_serve_data(mon, buf);
 		}
@@ -231,7 +233,8 @@ static GkrellmdMonitor cpu_monitor =
 	"cpu",
 	update_cpu,
 	serve_cpu_data,
-	serve_cpu_setup
+	serve_cpu_setup,
+	NULL
 	};
 
 
@@ -312,6 +315,7 @@ static GkrellmdMonitor proc_monitor =
 	"proc",
 	update_proc,
 	serve_proc_data,
+	NULL,
 	NULL
 	};
 
@@ -347,11 +351,10 @@ static gboolean	units_are_blocks;
 
 
 static DiskData *
-add_disk(gchar *name, gint order, gint device_number, gint unit_number)
+add_disk(const gchar *name, gint order, gint device_number, gint unit_number)
 	{
 	DiskData	*disk;
 	GList		*list;
-	gint		i;
 
 	disk = g_new0(DiskData, 1);
 	disk->name = g_strdup(name);
@@ -361,10 +364,14 @@ add_disk(gchar *name, gint order, gint device_number, gint unit_number)
 	disk->unit_number = unit_number;
 	if (order >= 0)
 		{
-		for (i = 0, list = disk_list; list; list = list->next, ++i)
+		int disk_pos = 0;
+		for (list = disk_list; list; list = list->next)
+			{
 			if (disk->order < ((DiskData *) list->data)->order)
 				break;
-		disk_list = g_list_insert(disk_list, disk, i);
+			++disk_pos;
+			}
+		disk_list = g_list_insert(disk_list, disk, disk_pos);
 		}
 	else
 		disk_list = g_list_append(disk_list, disk);
@@ -376,7 +383,6 @@ static DiskData *
 add_subdisk(gchar *subdisk_name, gchar *disk_name, gint subdisk)
 	{
 	DiskData	*sdisk = NULL;
-#if GLIB_CHECK_VERSION(2,0,0)
 	DiskData	*disk;
 	GList		*list = NULL;
 
@@ -402,7 +408,6 @@ add_subdisk(gchar *subdisk_name, gchar *disk_name, gint subdisk)
 		}
 	disk_list = g_list_insert_before(disk_list, list, sdisk);
 	++n_disks;
-#endif
 	return sdisk;
 	}
 
@@ -433,6 +438,19 @@ gkrellm_disk_units_are_blocks(void)
 	units_are_blocks = TRUE;
 	}
 
+void
+gkrellm_disk_add_by_name(const gchar *name, const gchar *label)
+	{
+	gint	order = -1;
+
+    if (NULL == name) // Cannot add disk without a name
+		return;
+    order = gkrellm_sys_disk_order_from_name(name);
+    /* FIXME: gkrellmd currently has no support for disk labels. Extend
+       network-protocol and server to support disks with both name and label. */
+    add_disk(name, order, 0, 0);
+	}
+
 void
 gkrellm_disk_assign_data_by_device(gint device_number, gint unit_number,
 			guint64 rb, guint64 wb, gboolean virtual)
@@ -471,7 +489,7 @@ gkrellm_disk_assign_data_nth(gint n, guint64 rb, guint64 wb, gboolean virtual)
 		disk = (DiskData *) g_list_nth_data(disk_list, n);
 	else
 		{
-		sprintf(name, "%s%c", _("Disk"), 'A' + n);
+		snprintf(name, sizeof(name), "%s%c", _("Disk"), 'A' + n);
 		disk = add_disk(name, n, 0, 0);
 		}
 	disk_assign_data(disk, rb, wb, virtual);
@@ -524,6 +542,8 @@ gkrellm_disk_subdisk_assign_data_by_name(gchar *subdisk_name, gchar *disk_name,
 		|  appended.  Eg. "hda1" is a subdisk_name of disk_name "hda"
 		*/
 		s = subdisk_name + strlen(disk_name);
+		if (*s == 'p')	/* except mmcblkN SD disks have "pN" partition numbers */
+			++s;
 		subdisk = strtol(s, &endptr, 0);
 		if (!*s || *endptr)
 			return;
@@ -556,7 +576,7 @@ serve_disk_data(GkrellmdMonitor *mon, gboolean first_serve)
 	{
 	DiskData	*disk;
 	GList		*list;
-	gchar		buf[128];
+	gchar		*buf = NULL;
 
 	gkrellmd_set_serve_name(mon, "disk");
 	for (list = disk_list; list; list = list->next)
@@ -567,18 +587,20 @@ serve_disk_data(GkrellmdMonitor *mon, gboolean first_serve)
 		if (!disk->subdisk_parent)
 			{
 			if (gkrellmd_check_client_version(mon, 2, 2, 7) && disk->virtual)
-				snprintf(buf, sizeof(buf), "%s virtual %" PRIu64 " %" PRIu64 "\n",
+                buf = g_strdup_printf("%s virtual %" PRIu64 " %" PRIu64 "\n",
 							disk->name, disk->rb, disk->wb);
 			else
-				snprintf(buf, sizeof(buf), "%s %" PRIu64 " %" PRIu64 "\n",
+				buf = g_strdup_printf("%s %" PRIu64 " %" PRIu64 "\n",
 							disk->name, disk->rb, disk->wb);
 			}
 		else if (mon->privat->client->feature_subdisk)
-			snprintf(buf, sizeof(buf), "%s %s %" PRIu64 " %" PRIu64 "\n",
+			buf = g_strdup_printf("%s %s %" PRIu64 " %" PRIu64 "\n",
 						disk->name, disk->subdisk_parent, disk->rb, disk->wb);
 		else
 			continue;
 		gkrellmd_serve_data(mon, buf);
+        g_free(buf);
+        buf = NULL;
 		}
 	}
 
@@ -598,7 +620,8 @@ static GkrellmdMonitor disk_monitor =
 	"disk",
 	update_disk,
 	serve_disk_data,
-	serve_disk_setup
+	serve_disk_setup,
+	NULL
 	};
 
 
@@ -681,7 +704,7 @@ update_inet(GkrellmdMonitor *mon, gboolean first_update)
 	InetData	*in;
 	static gint	check_tcp;
 
-	
+
 	if (!first_update && !GK.second_tick)
 		return;
 
@@ -723,8 +746,8 @@ serve_inet_data(GkrellmdMonitor *mon, gboolean first_serve)
 	InetData	*in;
 	ActiveTCP	*tcp;
 	GList		*list;
-	gchar		buf[128], *cp;
-#if defined(INET6) && defined(HAVE_GETADDRINFO)
+	gchar		buf[NI_MAXHOST + 128], *cp;
+#if defined(INET6)
 	struct sockaddr_in6	sin6;
 	char		addrbuf[NI_MAXHOST];
 #endif
@@ -741,10 +764,10 @@ serve_inet_data(GkrellmdMonitor *mon, gboolean first_serve)
 			   )
 				{
 				cp = inet_ntoa(tcp->remote_addr);
-				sprintf(buf, "+0 %x %s:%x\n",
+				snprintf(buf, sizeof(buf), "+0 %x %s:%x\n",
 							tcp->local_port, cp, tcp->remote_port);
 				}
-#if defined(INET6) && defined(HAVE_GETADDRINFO)
+#if defined(INET6)
 			else if (tcp->family == AF_INET6
 				 && (in->new_connection || first_serve))
 				{
@@ -783,10 +806,10 @@ serve_inet_data(GkrellmdMonitor *mon, gboolean first_serve)
 			if (tcp->family == AF_INET)
 				{
 				cp = inet_ntoa(tcp->remote_addr);
-				sprintf(buf, "-0 %x %s:%x\n",
+				snprintf(buf, sizeof(buf), "-0 %x %s:%x\n",
 							tcp->local_port, cp, tcp->remote_port);
 				}
-#if defined(INET6) && defined(HAVE_GETADDRINFO)
+#if defined(INET6)
 			else if (tcp->family == AF_INET6)
 				{
 				memset(&sin6, 0, sizeof(sin6));
@@ -830,7 +853,8 @@ static GkrellmdMonitor inet_monitor =
 	"inet",
 	update_inet,
 	serve_inet_data,
-	serve_inet_setup
+	serve_inet_setup,
+	NULL
 	};
 
 
@@ -1020,7 +1044,7 @@ update_net(GkrellmdMonitor *mon, gboolean first_update)
 			gkrellm_sys_net_check_routes();
 		}
 	gkrellm_sys_net_read_data();
-	
+
 	if (GK.second_tick && !net_use_routed)
 		{
 		for (list = net_list; list; list = list->next)
@@ -1037,13 +1061,13 @@ update_net(GkrellmdMonitor *mon, gboolean first_update)
 		{
 		if (net_timer_type == TIMER_TYPE_PPP)
 			{
-			struct stat		st;
+			GStatBuf		st;
 			gchar			buf[256];
 
 			if (net_timer->up_event)
 				{
 				snprintf(buf, sizeof(buf), "/var/run/%s.pid", net_timer->name);
-				if (stat(buf, &st) == 0)
+				if (g_stat(buf, &st) == 0)
 					net_timer0 = st.st_mtime;
 				else
 					time(&net_timer0);
@@ -1089,7 +1113,7 @@ serve_net_data(GkrellmdMonitor *mon, gboolean first_serve)
 		net = (NetData *) list->data;
 		if (net->changed || first_serve)
 			{
-			sprintf(buf, "%s %lu %lu\n", net->name, net->rx, net->tx);
+			snprintf(buf, sizeof(buf), "%s %lu %lu\n", net->name, net->rx, net->tx);
 			gkrellmd_serve_data(mon, buf);
 			}
 		}
@@ -1106,7 +1130,7 @@ serve_net_data(GkrellmdMonitor *mon, gboolean first_serve)
 			fake_up_event = (first_serve && net->up);
 			if (net->up_event || net->down_event || fake_up_event)
 				{
-				sprintf(buf, "%s %d\n", net->name,
+				snprintf(buf, sizeof(buf), "%s %d\n", net->name,
 						fake_up_event ? TRUE : net->up_event);
 				gkrellmd_serve_data(mon, buf);
 				}
@@ -1120,7 +1144,7 @@ serve_net_data(GkrellmdMonitor *mon, gboolean first_serve)
 		if (net_timer->timed_changed || first_serve)
 			{
 			gkrellmd_set_serve_name(mon, "net_timer");
-			sprintf(buf, "%s %d\n", net_timer->name, (gint)net_timer->up_time);
+			snprintf(buf, sizeof(buf), "%s %d\n", net_timer->name, (gint)net_timer->up_time);
 			gkrellmd_serve_data(mon, buf);
 			}
 		}
@@ -1151,7 +1175,8 @@ static GkrellmdMonitor net_monitor =
 	"net",
 	update_net,
 	serve_net_data,
-	serve_net_setup
+	serve_net_setup,
+	NULL
 	};
 
 
@@ -1266,6 +1291,7 @@ static GkrellmdMonitor mem_monitor =
 	"mem",
 	update_mem,
 	serve_mem_data,
+	NULL,
 	NULL
 	};
 
@@ -1292,7 +1318,7 @@ typedef struct
 				*device,
 				*type,
 				*options;
-	glong		blocks,
+	gint64		blocks,
 				bavail,
 				bfree,
 				bsize;
@@ -1311,13 +1337,15 @@ static gboolean	nfs_check,
 
 static gchar *remote_fs_types[] =
 	{
+	"cifs",
 	"nfs",
 	"smbfs"
 	};
 
 void
 gkrellm_fs_setup_eject(gchar *eject_tray, gchar *close_tray,
-			void (*eject_func)(), void (*close_func)())
+			GkrellmFsEjectFunc eject_func,
+			GkrellmFsCloseFunc close_func)
 	{
 	/* Not supported remotely */
 	}
@@ -1327,7 +1355,6 @@ gkrellm_fs_add_to_mounts_list(gchar *dir, gchar *dev, gchar *type)
 	{
 	GList	*list;
 	Mount	*m;
-	gint	i;
 
 	for (list = mounts_list; list; list = list->next)
 		{
@@ -1348,7 +1375,7 @@ gkrellm_fs_add_to_mounts_list(gchar *dir, gchar *dev, gchar *type)
 		mounts_list_modified = TRUE;
 		serveflag_done_list = g_list_append(serveflag_done_list, &m->changed);
 
-		for (i = 0; i < (sizeof(remote_fs_types) / sizeof(gchar *)); ++i)
+		for (size_t i = 0; i < G_N_ELEMENTS(remote_fs_types); ++i)
 			{
 			if (!strcmp(m->type, remote_fs_types[i]))
 				{
@@ -1374,7 +1401,7 @@ gkrellm_fs_add_to_fstab_list(gchar *dir, gchar *dev, gchar *type, gchar *opt)
 
 void
 gkrellm_fs_assign_fsusage_data(gpointer pointer,
-			glong blocks, glong bavail, glong bfree, glong bsize)
+		gint64 blocks, gint64 bavail, gint64 bfree, gint64 bsize)
 	{
 	Mount	*m = (Mount *) pointer;
 
@@ -1452,7 +1479,6 @@ refresh_fstab_list(void)
 	fstab_list_modified = TRUE;
 	}
 
-#if GLIB_CHECK_VERSION(2,0,0)
 static gpointer
 get_fsusage_thread(void *data)
 	{
@@ -1470,11 +1496,11 @@ get_fsusage_thread(void *data)
 		}
 	return NULL;
 	}
-#endif
 
 static void
 update_fs(GkrellmdMonitor *mon, gboolean first_update)
 	{
+	GThread		*gth;
 	GList		*list;
 	Mount		*m;
 	static gint	check_tick;
@@ -1502,15 +1528,12 @@ update_fs(GkrellmdMonitor *mon, gboolean first_update)
 			gkrellm_sys_fs_get_fsusage(m, m->directory);
 		else if (nfs_check && m->is_nfs && !m->busy)
 			{
-#if GLIB_CHECK_VERSION(2,0,0)
 			m->busy = TRUE;
-			g_thread_create(get_fsusage_thread, m, FALSE, NULL);
-#else
-			gkrellm_sys_fs_get_fsusage(m, m->directory);
-#endif
+			gth = g_thread_new("get_fsusage", get_fsusage_thread, m);
+			g_thread_unref(gth);
 			}
 		}
-	if (gkrellm_sys_fs_fstab_modified())
+	if (first_update || gkrellm_sys_fs_fstab_modified())
 		refresh_fstab_list();
 
 	gkrellmd_need_serve(mon);
@@ -1530,9 +1553,12 @@ serve_fs_data(GkrellmdMonitor *mon, gboolean first_serve)
 		for (list = mounts_list; list; list = list->next)
 			{
 			m = (Mount *) list->data;
-			snprintf(buf, sizeof(buf), "%s %s %s %ld %ld %ld %ld\n",
+			snprintf(buf, sizeof(buf),
+				"%s %s %s %" PRIu64 " %" PRIu64 " %" PRIu64 " %" PRIu64 "\n",
 					m->directory, m->device, m->type,
 					m->blocks, m->bavail, m->bfree, m->bsize);
+			/*gkrellm_debug(DEBUG_SERVER,
+				"Adding mount-line for %s to serve-data\n", m->directory);*/
 			gkrellmd_serve_data(mon, buf);
 			}
 		}
@@ -1544,9 +1570,12 @@ serve_fs_data(GkrellmdMonitor *mon, gboolean first_serve)
 			m = (Mount *) list->data;
 			if (!m->changed)
 				continue;
-			snprintf(buf, sizeof(buf), "%s %s %ld %ld %ld %ld\n",
+			snprintf(buf, sizeof(buf),
+				"%s %s %" PRIu64 " %" PRIu64 " %" PRIu64 " %" PRIu64 "\n",
 					m->directory, m->device,
 					m->blocks, m->bavail, m->bfree, m->bsize);
+			/*gkrellm_debug(DEBUG_SERVER,
+				"Updating fs %s in serve-data\n", m->directory);*/
 			gkrellmd_serve_data(mon, buf);
 			}
 		}
@@ -1559,6 +1588,8 @@ serve_fs_data(GkrellmdMonitor *mon, gboolean first_serve)
 			m = (Mount *) list->data;
 			snprintf(buf, sizeof(buf), "%s %s %s\n",
 					m->directory, m->device, m->type);
+			/*gkrellm_debug(DEBUG_SERVER,
+				"Adding fstab-line for %s to serve-data\n", m->directory);*/
 			gkrellmd_serve_data(mon, buf);
 			}
 		}
@@ -1578,7 +1609,8 @@ static GkrellmdMonitor fs_monitor =
 	"fs",
 	update_fs,
 	serve_fs_data,
-	serve_fs_setup
+	serve_fs_setup,
+	NULL
 	};
 
 
@@ -1681,7 +1713,7 @@ gkrellm_battery_assign_data(gint id, gboolean present, gboolean on_line,
 	}
 
 gint
-gkrellm_battery_full_cap_fallback()
+gkrellm_battery_full_cap_fallback(void)
 	{
 	return 5000;	/* XXX Linux ACPI bug not handled by server */
 	}
@@ -1757,7 +1789,8 @@ static GkrellmdMonitor battery_monitor =
 	"battery",
 	update_battery,
 	serve_battery_data,
-	serve_battery_setup
+	serve_battery_setup,
+	NULL
 	};
 
 static GkrellmdMonitor *
@@ -1818,6 +1851,9 @@ read_sensors(void *data)
 		else if (sensor->type == SENSOR_VOLTAGE)
 			gkrellm_sys_sensors_get_voltage(sensor->path, sensor->id,
 				sensor->iodev, sensor->inter, &sensor->raw_value);
+		else if (sensor->type == SENSOR_POWER)
+			gkrellm_sys_sensors_get_power(sensor->path, sensor->id,
+				sensor->iodev, sensor->inter, &sensor->raw_value);
 		if (sensor->raw_value != tmp)
 			{
 			sensor->changed = TRUE;
@@ -1835,24 +1871,20 @@ read_sensors(void *data)
 static void
 run_sensors_thread(void)
 	{
-#if GLIB_CHECK_VERSION(2,0,0)
+	GThread		*gth;
+
 	if (thread_busy)
 		return;
 	thread_busy = TRUE;
-	g_thread_create(read_sensors, NULL, FALSE, NULL);
-#else
-	read_sensors(NULL);
-#endif
+	gth = g_thread_new("read_sensors", read_sensors, NULL);
+	g_thread_unref(gth);
 	}
 
 
 void
-gkrellm_sensors_config_migrate_connect(gboolean (*func)(), gint sysdep_version)
-	{
-	}
-
-void
-gkrellm_sensors_update_volt_order_base(void)
+gkrellm_sensors_config_migrate_connect(
+			GkrellmSensorsConfigMigrateFunc migrate_func,
+			gint sysdep_private)
 	{
 	}
 
@@ -1866,7 +1898,8 @@ gkrellm_sensors_set_group(gpointer sr, gint group)
 	}
 
 void
-gkrellm_sensors_sysdep_option(gchar *keyword, gchar *label, void (*func)())
+gkrellm_sensors_sysdep_option(gchar *keyword, gchar *label,
+			GkrellmSensorsSysdepOptionFunc func)
 	{
 	}
 
@@ -1940,7 +1973,7 @@ serve_sensors_data(GkrellmdMonitor *mon, gboolean first_serve)
 			continue;
 		if (sr->changed || first_serve)
 			{
-			sprintf(buf, "%d \"%s\" %d %d %d %.2f\n",
+			snprintf(buf, sizeof(buf), "%d \"%s\" %d %d %d %.2f\n",
 					sr->type, sr->id_name,
 					sr->id, sr->iodev, sr->inter, sr->raw_value);
 			gkrellmd_serve_data(mon, buf);
@@ -1965,12 +1998,12 @@ serve_sensors_setup(GkrellmdMonitor *mon)
 		if (s->group == SENSOR_GROUP_DISK && !sensor_disk_ok)
 			continue;
 		if (sensor_disk_ok)
-			sprintf(buf, "%d \"%s\" %d %d %d %.4f %.4f \"%s\" \"%s\" %d\n",
+			snprintf(buf, sizeof(buf), "%d \"%s\" %d %d %d %.4f %.4f \"%s\" \"%s\" %d\n",
 					s->type, s->id_name,
 					s->id, s->iodev, s->inter,
 					s->factor, s->offset, s->vref, s->default_label, s->group);
 		else
-			sprintf(buf, "%d \"%s\" %d %d %d %.4f %.4f \"%s\" \"%s\"\n",
+			snprintf(buf, sizeof(buf), "%d \"%s\" %d %d %d %.4f %.4f \"%s\" \"%s\"\n",
 					s->type, s->id_name,
 					s->id, s->iodev, s->inter,
 					s->factor, s->offset, s->vref, s->default_label);
@@ -1983,7 +2016,8 @@ static GkrellmdMonitor sensors_monitor =
 	"sensors",
 	update_sensors,
 	serve_sensors_data,
-	serve_sensors_setup
+	serve_sensors_setup,
+	NULL
 	};
 
 static GkrellmdMonitor *
@@ -1997,7 +2031,7 @@ init_sensors_monitor(void)
 /* ======================================================= */
 static time_t	base_uptime,
 				up_seconds;
-static gulong	up_minutes = -1;
+static glong	up_minutes = -1;
 
 void
 gkrellm_uptime_set_base_uptime(time_t base)
@@ -2008,16 +2042,16 @@ gkrellm_uptime_set_base_uptime(time_t base)
 static void
 update_uptime(GkrellmdMonitor *mon, gboolean first_update)
 	{
-	gint	prev_up;
+	glong	prev_up;
 
 	if (GK.ten_second_tick || up_minutes < 0 || first_update)
 		{
 		prev_up = up_minutes;
 		up_seconds = gkrellm_sys_uptime_read_uptime();
 		if (up_seconds > 0)
-			up_minutes = (gint) (up_seconds / 60);
+			up_minutes = (glong) (up_seconds / 60);
 		else
-			up_minutes = (gint)(time(0) - _GK.start_time + base_uptime) / 60;
+			up_minutes = (glong)(time(0) - _GK.start_time + base_uptime) / 60;
 		if (up_minutes != prev_up)
 			gkrellmd_need_serve(mon);
 		}
@@ -2029,7 +2063,7 @@ serve_uptime_data(GkrellmdMonitor *mon, gboolean first_serve)
 	gchar	buf[128];
 
 	gkrellmd_set_serve_name(mon, "uptime");
-	snprintf(buf, sizeof(buf), "%lu\n", (gulong) up_minutes);
+	snprintf(buf, sizeof(buf), "%ld\n", (glong) (up_minutes >= 0 ? up_minutes : 0));
 	gkrellmd_serve_data(mon, buf);
 	}
 
@@ -2038,6 +2072,7 @@ static GkrellmdMonitor uptime_monitor =
 	"uptime",
 	update_uptime,
 	serve_uptime_data,
+	NULL,
 	NULL
 	};
 
@@ -2057,7 +2092,7 @@ send_time(GkrellmdClient *client)
 	gchar		buf[128];
 
 	t = &gkrellmd_current_tm;
-	sprintf(buf, "<time>\n%d %d %d %d %d %d %d %d %d\n",
+	snprintf(buf, sizeof(buf), "<time>\n%d %d %d %d %d %d %d %d %d\n",
 		t->tm_sec, t->tm_min, t->tm_hour,
 		t->tm_mday, t->tm_mon, t->tm_year,
 		t->tm_wday, t->tm_yday, t->tm_isdst);
@@ -2111,7 +2146,7 @@ gkrellmd_load_monitors(void)
 	if (_GK.list_plugins)
 		exit(0);
 	if (_GK.log_plugins)
-		printf("%s\n", plugin_install_log ? plugin_install_log :
+		g_message("%s\n", plugin_install_log ? plugin_install_log :
 				_("No plugins found\n"));
 	for (  ; list; list = list->next)
 		{
@@ -2196,12 +2231,12 @@ gkrellmd_update_monitors(void)
 			}
 		gkrellmd_send_to_client(client, serve_gstring->str);
 		serve_gstring = g_string_truncate(serve_gstring, 0);
-		
+
 		if (GK.minute_tick || !client->served)
 			send_time(client);
 		else if (GK.second_tick)
 			{
-			sprintf(buf, "<.%d>\n", gkrellmd_current_tm.tm_sec);
+			snprintf(buf, sizeof(buf), "<.%d>\n", gkrellmd_current_tm.tm_sec);
 			gkrellmd_send_to_client(client, buf);
 			}
 
@@ -2234,14 +2269,14 @@ gkrellmd_serve_setup(GkrellmdClient *client)
 
 	gkrellmd_send_to_client(client, "<gkrellmd_setup>\n");
 
-	s = g_strdup_printf("<version>\ngkrellmd %d.%d.%d%s\n",
+	s = g_strdup_printf("<version>\ngkrellmd %d.%d.%d\n",
 				GKRELLMD_VERSION_MAJOR, GKRELLMD_VERSION_MINOR,
-				GKRELLMD_VERSION_REV, GKRELLMD_EXTRAVERSION);
+				GKRELLMD_VERSION_REV);
 	gkrellmd_send_to_client(client, s);
 	g_free(s);
 
 	lc = localeconv();
-	sprintf(buf, "%c\n", *lc->decimal_point);
+	snprintf(buf, sizeof(buf), "%c\n", *lc->decimal_point);
 	s = g_strconcat("<decimal_point>\n", buf, "\n", NULL);
 	gkrellmd_send_to_client(client, s);
 	g_free(s);
@@ -2269,16 +2304,16 @@ gkrellmd_serve_setup(GkrellmdClient *client)
 	for (list = gkrellmd_monitor_list; list; list = list->next)
 		{
 		mon = (GkrellmdMonitor *) list->data;
-		sprintf(buf, "%s\n", mon->name);
+		snprintf(buf, sizeof(buf), "%s\n", mon->name);
 		gkrellmd_send_to_client(client, buf);
 		}
 
-	sprintf(buf, "%d\n", _GK.io_timeout);
+	snprintf(buf, sizeof(buf), "%d\n", _GK.io_timeout);
 	s = g_strconcat("<io_timeout>\n", buf, "\n", NULL);
 	gkrellmd_send_to_client(client, s);
 	g_free(s);
 
-	sprintf(buf, "%d\n", _GK.reconnect_timeout);
+	snprintf(buf, sizeof(buf), "%d\n", _GK.reconnect_timeout);
 	s = g_strconcat("<reconnect_timeout>\n", buf, "\n", NULL);
 	gkrellmd_send_to_client(client, s);
 	g_free(s);
