@@ -1,8 +1,8 @@
 /* GKrellM
-|  Copyright (C) 1999-2014 Bill Wilson
+|  Copyright (C) 1999-2021 Bill Wilson
 |
-|  Author:  Bill Wilson    bill@gkrellm.net
-|  Latest versions might be found at:  http://gkrellm.net
+|  Author:  Bill Wilson
+|  Latest versions might be found at:  https://gkrellm.srcbox.net
 |
 |  DragonFly code: Joerg Sonnenberger <joerg@bec.de>
 |  Derived from FreeBSD code: Hajimu UMEMOTO <ume@FreeBSD.org>
@@ -17,7 +17,7 @@
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 |  GNU General Public License for more details.  Version 2 is in the
 |  COPYRIGHT file in the top level directory of this distribution.
-| 
+|
 |  To get a copy of the GNU General Puplic License, write to the Free Software
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
@@ -28,7 +28,6 @@ kvm_t	*kvmd = NULL;
 char	errbuf[_POSIX2_LINE_MAX];
 
 
-// extern gboolean force_meminfo_update(void);
 #if defined(__i386__)
 static void scan_for_sensors();
 #endif
@@ -458,7 +457,7 @@ swapmode(guint64 *retavail, guint64 *retfree)
 	*retfree = avail - used;
 
 	/* increase counter, no errors occurs */
-	warning++; 
+	warning++;
 
 	return  (int)(((double)used / (double)avail * 100.0) + 0.5);
 	}
@@ -485,7 +484,7 @@ get_bufspace(guint64 *bufspacep)
 		   &bufspace, &bufspace_len, 0, 0) < 0)
 		return 0;
 	*bufspacep = bufspace;
-	return 1;	
+	return 1;
 	}
 
 struct mibtab {
@@ -540,15 +539,6 @@ gkrellm_sys_mem_read_data(void)
 	FILE		*f;
 	gchar		buf[160];
 
-#if 0
-	/* mem.c does a force_meminfo_update() before calling this */
-	/* Collecting meminfo data is expensive under FreeBSD, so
-	|  take extra precautions to minimize reading it.
-	*/
-	if (!GK.ten_second_tick && !force_meminfo_update())
-		return;
-#endif
-
 	if (pshift == 0)
 		{
 		for (psize = getpagesize(); psize > 1; psize >>= 1)
@@ -968,6 +958,13 @@ gkrellm_sys_sensors_get_voltage(gchar *path, gint id,
 	return FALSE;
 	}
 
+gboolean
+gkrellm_sys_sensors_get_power(gchar *path, gint id,
+		gint iodev, gint interface, gfloat *power)
+	{
+	return FALSE;
+	}
+
 struct freebsd_sensor {
 	gint type;
 	gchar *id_name;
@@ -1167,6 +1164,13 @@ gkrellm_sys_sensors_get_voltage(gchar *path, gint id,
 	return FALSE;
 	}
 
+gboolean
+gkrellm_sys_sensors_get_power(gchar *path, gint id,
+		gint iodev, gint interface, gfloat *power)
+	{
+	return FALSE;
+	}
+
 gboolean
 gkrellm_sys_sensors_init(void)
 	{
