@@ -1,8 +1,8 @@
 /* GKrellM
-|  Copyright (C) 1999-2010 Bill Wilson
+|  Copyright (C) 1999-2021 Bill Wilson
 |
-|  Author:  Bill Wilson    billw@gkrellm.net
-|  Latest versions might be found at:  http://gkrellm.net
+|  Author:  Bill Wilson
+|  Latest versions might be found at:  https://gkrellm.srcbox.net
 |
 |  freebsd.c code is Copyright (C) Hajimu UMEMOTO <ume@FreeBSD.org>
 |
@@ -33,19 +33,16 @@
 |  as that of the covered work.
 */
 
+// Drop support for FreeBSD older than ~10 years regularly and bump this number
+#if __FreeBSD_version < 1000000
+#error Requires FreeBSD 10.0 or newer
+#endif
+
 #include <sys/param.h>
 #include <sys/sysctl.h>
 #include <osreldate.h>
 
-#if __FreeBSD_version < 500000
-#include <kvm.h>
 
-kvm_t	*kvmd = NULL;
-char	errbuf[_POSIX2_LINE_MAX];
-#endif
-
-
-// extern gboolean force_meminfo_update(void);
 #if defined(__i386__) || defined(__amd64__)
 static void scan_for_sensors();
 #endif
@@ -57,9 +54,6 @@ gkrellm_sys_main_init(void)
 	/* We just ignore error, here.  Even if GKrellM doesn't have
 	|  kmem privilege, it runs with available information.
 	*/
-#if __FreeBSD_version < 500000
-	kvmd = kvm_openfiles(NULL, NULL, NULL, O_RDONLY, errbuf);
-#endif
 	if (setgid(getgid()) != 0)
 		{
 		fprintf(stderr, "Can't drop setgid privileges.");
@@ -95,11 +89,7 @@ gk_sysctlnametomib(const char *name, int *mibp, size_t *lenp)
 /* ===================================================================== */
 /* CPU monitor interface */
 
-#if __FreeBSD_version >= 500101
 #include <sys/resource.h>
-#else
-#include <sys/dkstat.h>
-#endif
 
 static int	oid_cp_time[CTL_MAXNAME + 2];
 static int	oid_cp_times[CTL_MAXNAME + 2];
@@ -116,13 +106,6 @@ gkrellm_sys_cpu_read_data(void)
 	{
 	long		cp_time[CPUSTATES], *cp_timep;
 	size_t		len;
-#if __FreeBSD_version < 500000
-	static struct nlist nl[] = {
-#define N_CP_TIME	0
-		{ "_cp_time" },
-		{ "" }
-	};
-#endif
 
 	if (have_cp_time)
 		{
@@ -131,19 +114,6 @@ gkrellm_sys_cpu_read_data(void)
 			   NULL, 0) < 0)
 			return;
 		}
-#if __FreeBSD_version < 500000
-	else
-		{
-		if (kvmd == NULL)
-			return;
-		if (nl[0].n_type == 0)
-			if (kvm_nlist(kvmd, nl) < 0 || nl[0].n_type == 0)
-				return;
-		if (kvm_read(kvmd, nl[N_CP_TIME].n_value, (char *)&cp_time,
-			     sizeof(cp_time)) != sizeof(cp_time))
-			return;
-		}
-#endif
 
 	if (ncpus > 1)
 		{
@@ -236,30 +206,11 @@ pcpu_probe_done:
 /* ===================================================================== */
 /* Proc monitor interface */
 
-#if __FreeBSD_version >= 400000
 #include <sys/user.h>
-#endif
-
-/*
- * This is ugly, but we need PID_MAX, in anyway.  Since 5.0-RELEASE
- * will have vm.stats.vm.v_forks, this will be obsolete in the future.
- */
-#if __FreeBSD_version >= 400000
-#define	PID_MAX		99999
-#else
-#include <sys/signalvar.h>
-#define KERNEL
-#include <sys/proc.h>
-#undef KERNEL
-#endif
 
 #include <limits.h>
 #include <paths.h>
-#if __FreeBSD_version < 900007
-#include <utmp.h>
-#else
 #include <utmpx.h>
-#endif
 
 static int	oid_v_forks[CTL_MAXNAME + 2];
 static int	oid_v_vforks[CTL_MAXNAME + 2];
@@ -290,23 +241,12 @@ gkrellm_sys_proc_init(void)
 void
 gkrellm_sys_proc_read_data(void)
 	{
-#if __FreeBSD_version >= 400000
 	static int	oid_proc[] = { CTL_KERN, KERN_PROC, KERN_PROC_ALL };
-#endif
 	double		avenrun;
 	static u_int	n_processes, n_forks = 0;
 	u_int		n_vforks, n_rforks;
 	gint		r_forks, r_vforks, r_rforks;
 	size_t		len;
-#if __FreeBSD_version < 500000
-	static u_int	curpid = -1;
-	gint		nextpid, nforked;
-	static struct nlist nl[] = {
-#define N_NEXTPID	0
-		{ "_nextpid" },
-		{ "" }
-	};
-#endif
 
 	if (getloadavg(&avenrun, 1) <= 0)
 		avenrun = 0;
@@ -327,38 +267,9 @@ gkrellm_sys_proc_read_data(void)
 		if (r_forks >= 0 && r_vforks >= 0 && r_rforks >= 0)
 			n_forks = n_forks + n_vforks + n_rforks;
 		}
-#if __FreeBSD_version < 500000
-	else
-		{
-		/* workaround: Can I get total number of processes? */
-		if (kvmd != NULL)
-			{
-			if (nl[0].n_type == 0)
-				kvm_nlist(kvmd, nl);
-			if (nl[0].n_type != 0 &&
-			    kvm_read(kvmd, nl[N_NEXTPID].n_value,
-				     (char *)&nextpid,
-				     sizeof(nextpid)) == sizeof(nextpid))
-				{
-				if (curpid < 0)
-					curpid = nextpid;
-				if ((nforked = nextpid - curpid) < 0)
-					n_forks += PID_MAX - 100;
-				n_forks += nforked;
-				curpid = nextpid;
-				n_forks = n_forks;
-				}
-			}
-		}
-#endif
 
-#if __FreeBSD_version >= 400000
 	if (sysctl(oid_proc, 3, NULL, &len, NULL, 0) >= 0)
 		n_processes = len / sizeof(struct kinfo_proc);
-#else
-	if (kvmd != NULL)
-		kvm_getprocs(kvmd, KERN_PROC_ALL, 0, &n_processes);
-#endif
 	gkrellm_proc_assign_data(n_processes, 0, n_forks, avenrun);
 	}
 
@@ -422,10 +333,8 @@ gkrellm_sys_proc_read_users(void)
 /* ===================================================================== */
 /* Disk monitor interface */
 
-#if __FreeBSD_version >= 300000
 #include <devstat.h>
 static struct statinfo	statinfo_cur;
-#endif
 
 gchar *
 gkrellm_sys_disk_name_from_device(gint device_number, gint unit_number,
@@ -434,62 +343,10 @@ gkrellm_sys_disk_name_from_device(gint device_number, gint unit_number,
 	return NULL;	/* Not implemented */
 	}
 
-#if __FreeBSD_version < 300000
-gint
-gkrellm_sys_disk_order_from_name(const gchar *name)
-	{
-	return 0;	/* Not implemented */
-	}
-
-void
-gkrellm_sys_disk_read_data(void)
-	{
-	int		ndevs;
-	long		*cur_dk_xfer;
-	int		dn;
-	GList		*list;
-	static struct nlist nl[] = {
-#define N_DK_NDRIVE	0
-		{ "_dk_ndrive" },
-#define N_DK_XFER	1
-		{ "_dk_xfer" },
-		{ "" }
-	};
-   
-	if (kvmd == NULL)
-		return;
-	if (kvm_nlist(kvmd, nl) < 0 || nl[0].n_type == 0)
-		return;
-	(void) kvm_read(kvmd, nl[N_DK_NDRIVE].n_value,
-			(char *)&ndevs, sizeof(ndevs));
-	if (ndevs <= 0)
-		return;
-	if ((cur_dk_xfer = calloc(ndevs, sizeof(long))) == NULL)
-		return;
-	if (kvm_read(kvmd, nl[N_DK_XFER].n_value, (char *)cur_dk_xfer,
-		     ndevs * sizeof(long)) == ndevs * sizeof(long))
-		{
-		for (dn = 0; dn < ndevs; ++dn)
-			gkrellm_disk_assign_data_nth(dn, 0, cur_dk_xfer[dn], FALSE);
-		}
-	free(cur_dk_xfer);
-	}
-
-gboolean
-gkrellm_sys_disk_init(void)
-	{
-	gkrellm_disk_units_are_blocks();
-	return TRUE;
-	}
-
-#else
-
-#if __FreeBSD_version >= 500107
 #define getdevs(stats)	devstat_getdevs(NULL, stats)
 #define selectdevs	devstat_selectdevs
 #define bytes_read	bytes[DEVSTAT_READ]
 #define bytes_written	bytes[DEVSTAT_WRITE]
-#endif
 
 gint
 gkrellm_sys_disk_order_from_name(const gchar *name)
@@ -551,7 +408,6 @@ gkrellm_sys_disk_init(void)
 	bzero(statinfo_cur.dinfo, sizeof(struct devinfo));
 	return TRUE;
 	}
-#endif
 
 
 /* ===================================================================== */
@@ -559,14 +415,10 @@ gkrellm_sys_disk_init(void)
 
 #include "../inet.h"
 
-#if defined(INET6)
 #include <netinet6/ip6_var.h>
-#endif
 #include <netinet/in_pcb.h>
 #include <netinet/ip_var.h>
-#if defined(INET6)
 #include <netinet/ip6.h>
-#endif
 #include <netinet/tcp.h>
 #include <netinet/tcpip.h>
 #include <netinet/tcp_seq.h>
@@ -574,102 +426,7 @@ gkrellm_sys_disk_init(void)
 #include <netinet/tcp_timer.h>
 #include <netinet/tcp_var.h>
 
-#if __FreeBSD_version < 300000
-#if defined(INET6)
-#include <netinet6/tcp6.h>
-#include <netinet6/in6_pcb.h>
-#include <netinet6/tcp6_timer.h>
-#include <netinet6/tcp6_var.h>
-#endif
 
-#include <nlist.h>
-#endif
-
-
-#if __FreeBSD_version < 300000
-void
-gkrellm_sys_inet_read_tcp_data(void)
-	{
-	ActiveTCP	tcp;
-	gint		tcp_status;
-	struct		inpcbhead head;
-	struct		inpcb inpcb, *next;
-#if defined(INET6)
-	struct		in6pcb in6pcb, *prev6, *next6;
-	u_long		off = nl[N_TCB6].n_value;
-#endif
-	static struct nlist nl[] = {
-#define	N_TCB		0
-		{ "_tcb" },
-#if defined(INET6)
-#define	N_TCB6		1
-		{ "_tcb6" },
-#endif
-		{ "" },
-	};
-
-
-	if (kvmd == NULL)
-		return;
-	if (nl[0].n_type == 0)
-		if (kvm_nlist(kvmd, nl) < 0 || nl[0].n_type == 0)
-			return;
-	if (kvm_read(kvmd, nl[N_TCB].n_value, (char *)&head,
-		     sizeof(struct inpcbhead)) != sizeof(struct inpcbhead))
-		return;
-
-	for (next = head.lh_first; next != NULL;
-	     next = inpcb.inp_list.le_next)
-		{
-		struct tcpcb tcpcb;
-
-		if (kvm_read(kvmd, (u_long)next, (char *)&inpcb,
-			     sizeof(inpcb)) != sizeof(inpcb))
-			return;
-		if (kvm_read(kvmd, (u_long)inpcb.inp_ppcb, (char *)&tcpcb,
-			     sizeof(tcpcb)) != sizeof(tcpcb))
-			return;
-		tcp.local_port = ntohs(inpcb.inp_lport);
-		tcp.remote_addr.s_addr = inpcb.inp_faddr.s_addr;
-		tcp.remote_port = ntohs(inpcb.inp_fport);
-		tcp_status = (tcpcb.t_state == TCPS_ESTABLISHED);
-		tcp.family = AF_INET;
-		if (tcp_status == TCP_ALIVE)
-			gkrellm_inet_log_tcp_port_data(&tcp);
-		}
-#if defined(INET6)
-	if (kvm_read(kvmd, off, (char *)&in6pcb,
-		         sizeof(struct in6pcb)) != sizeof(struct in6pcb))
-		return;
-	prev6 = (struct in6pcb *)off;
-	if (in6pcb.in6p_next == (struct in6pcb *)off)
-		return;
-	while (in6pcb.in6p_next != (struct in6pcb *)off)
-		{
-		struct tcp6cb tcp6cb;
-
-		next6 = in6pcb.in6p_next;
-		if (kvm_read(kvmd, (u_long)next6, (char *)&in6pcb,
-			     sizeof(in6pcb)) != sizeof(in6pcb))
-			return;
-		if (in6pcb.in6p_prev != prev6)
-			break;
-		if (kvm_read(kvmd, (u_long)in6pcb.in6p_ppcb, (char *)&tcp6cb,
-			     sizeof(tcp6cb)) != sizeof(tcp6cb))
-			return;
-		tcp.local_port = ntohs(in6pcb.in6p_lport);
-		memcpy(&tcp.remote_addr6, &in6pcb.in6p_faddr,
-		       sizeof(struct in6_addr));
-		tcp.remote_port = ntohs(in6pcb.in6p_fport);
-		tcp_status = (tcp6cb.t_state == TCPS_ESTABLISHED);
-		tcp.family = AF_INET6;
-		if (tcp_status == TCP_ALIVE)
-			gkrellm_inet_log_tcp_port_data(&tcp);
-		prev6 = next6;
-		}
-#endif
-	}
-#else
 void
 gkrellm_sys_inet_read_tcp_data(void)
 	{
@@ -701,9 +458,15 @@ gkrellm_sys_inet_read_tcp_data(void)
 		     xig->xig_len > sizeof(struct xinpgen);
 		     xig = (struct xinpgen *)((char *)xig + xig->xig_len))
 			{
+#if __FreeBSD_version >= 1200026
+			struct xtcpcb *tp = (struct xtcpcb *)xig;
+			struct xinpcb *inp = &tp->xt_inp;
+			struct xsocket *so = &inp->xi_socket;
+#else
 			struct tcpcb *tp = &((struct xtcpcb *)xig)->xt_tp;
 			struct inpcb *inp = &((struct xtcpcb *)xig)->xt_inp;
 			struct xsocket *so = &((struct xtcpcb *)xig)->xt_socket;
+#endif
 
 			/* Ignore sockets for protocols other than tcp. */
 			if (so->xso_protocol != IPPROTO_TCP)
@@ -713,7 +476,6 @@ gkrellm_sys_inet_read_tcp_data(void)
 			if (inp->inp_gencnt > oxig->xig_gen)
 				continue;
 
-#if defined(INET6)
 			if (inp->inp_vflag & INP_IPV4)
 				{
 				tcp.remote_addr.s_addr = inp->inp_faddr.s_addr;
@@ -728,10 +490,6 @@ gkrellm_sys_inet_read_tcp_data(void)
 				}
 			else
 				continue;
-#else
-			tcp.remote_addr.s_addr = inp->inp_faddr.s_addr;
-			tcp.family = AF_INET;
-#endif
 			tcp.remote_port = ntohs(inp->inp_fport);
 			tcp.local_port = ntohs(inp->inp_lport);
 			tcp_status = (tp->t_state == TCPS_ESTABLISHED);
@@ -741,7 +499,6 @@ gkrellm_sys_inet_read_tcp_data(void)
 		}
 	free(buf);
 	}
-#endif
 
 
 gboolean
@@ -755,63 +512,18 @@ gkrellm_sys_inet_init(void)
 /* Memory/Swap monitor interface */
 
 #include <sys/conf.h>
-#if __FreeBSD_version < 400000
-#include <sys/rlist.h>
-#endif
 #include <sys/vmmeter.h>
 #include <vm/vm_param.h>
 
-#if __FreeBSD_version < 410000
-static struct nlist nl_mem[] = {
-#define N_CNT		0
-	{ "_cnt" },
-#if __FreeBSD_version < 400000
-#define VM_SWAPLIST	1
-	{ "_swaplist" },
-#define VM_SWDEVT	2
-	{ "_swdevt" },
-#define VM_NSWAP	3
-	{ "_nswap" },
-#define VM_NSWDEV	4
-	{ "_nswdev" },
-#define VM_DMMAX	5
-	{ "_dmmax" },
-#if  __FreeBSD_version < 300000
-#define N_BUFSPACE	6
-	{ "_bufspace" },
-#endif
-#endif
-	{ "" }
-};
-#endif
-
 static int
 swapmode(unsigned long long *retavail, unsigned long long *retfree)
 	{
 	guint64 used, avail;
-#if  __FreeBSD_version >= 400000
 	static int psize = -1;
 	struct kvm_swap kvmswap;
-#if __FreeBSD_version >= 500000
 	struct xswdev xsw;
 	size_t mibsize, size;
 	int mib[16], n;
-#endif
-#else
-	char *header;
-	int hlen, nswap, nswdev, dmmax;
-	int i, div, nfree, npfree;
-	struct swdevt *sw;
-	long blocksize, *perdev;
-	u_long ptr;
-	struct rlist head;
-#  if __FreeBSD_version >= 220000
-	struct rlisthdr swaplist;
-#  else 
-	struct rlist *swaplist;
-#  endif
-	struct rlist *swapptr;
-#endif
 
 	/*
 	 * Counter for error messages. If we reach the limit,
@@ -833,8 +545,6 @@ swapmode(unsigned long long *retavail, unsigned long long *retfree)
 		}
 	warning--;		/* decrease counter, see end of function */
 
-#if __FreeBSD_version >= 400000
-#if __FreeBSD_version >= 500000
 	mibsize = sizeof mib / sizeof mib[0];
 	if (gk_sysctlnametomib("vm.swap_info", mib, &mibsize) == -1)
 		return(0);
@@ -849,140 +559,14 @@ swapmode(unsigned long long *retavail, unsigned long long *retfree)
 		kvmswap.ksw_total += xsw.xsw_nblks;
 		kvmswap.ksw_used += xsw.xsw_used;
 		}
-#else
-	if (kvmd == NULL)
-		return(0);
-	if (kvm_getswapinfo(kvmd, &kvmswap, 1, 0) < 0)
-		return(0);
-
-#endif
 	if (psize < 0)
 		psize = getpagesize();
 	*retavail = avail = (quad_t)kvmswap.ksw_total * psize;
 	used = (quad_t)kvmswap.ksw_used * psize;
 	*retfree = avail - used;
-#else
-	if (kvmd == NULL)
-		return(0);
-	if (kvm_read(kvmd, nl_mem[VM_NSWAP].n_value,
-		     &nswap, sizeof(nswap)) != sizeof(nswap))
-		return(0);
-	if (!nswap)
-		{
-		fprintf(stderr, "No swap space available\n");
-		return(0);
-		}
-
-	if (kvm_read(kvmd, nl_mem[VM_NSWDEV].n_value,
-		     &nswdev, sizeof(nswdev)) != sizeof(nswdev))
-		return(0);
-	if (kvm_read(kvmd, nl_mem[VM_DMMAX].n_value,
-		     &dmmax, sizeof(dmmax)) != sizeof(dmmax))
-		return(0);
-	if (kvm_read(kvmd, nl_mem[VM_SWAPLIST].n_value,
-		     &swaplist, sizeof(swaplist)) != sizeof(swaplist))
-		return(0);
-
-	if ((sw = (struct swdevt *)malloc(nswdev * sizeof(*sw))) == NULL ||
-	    (perdev = (long *)malloc(nswdev * sizeof(*perdev))) == NULL)
-		{
-		perror("malloc");
-		exit(1);
-		}
-	if (kvm_read(kvmd, nl_mem[VM_SWDEVT].n_value,
-		     &ptr, sizeof ptr) != sizeof ptr)
-		return(0);
-	if (kvm_read(kvmd, ptr,
-		     sw, nswdev * sizeof(*sw)) != nswdev * sizeof(*sw))
-		return(0);
-
-	/* Count up swap space. */
-	nfree = 0;
-	memset(perdev, 0, nswdev * sizeof(*perdev));
-#if  __FreeBSD_version >= 220000
-	swapptr = swaplist.rlh_list;
-	while (swapptr)
-#else
-	while (swaplist)
-#endif
-		{
-		int	top, bottom, next_block;
-#if  __FreeBSD_version >= 220000
-		if (kvm_read(kvmd, (u_long)swapptr, &head,
-			     sizeof(struct rlist)) != sizeof(struct rlist))
-			return (0);
-#else
-		if (kvm_read(kvmd, (u_long)swaplist, &head,
-			     sizeof(struct rlist)) != sizeof(struct rlist))
-			return (0);
-#endif
-
-		top = head.rl_end;
-		bottom = head.rl_start;
-
-		nfree += top - bottom + 1;
-
-		/*
-		 * Swap space is split up among the configured disks.
-		 *
-		 * For interleaved swap devices, the first dmmax blocks
-		 * of swap space some from the first disk, the next dmmax
-		 * blocks from the next, and so on up to nswap blocks.
-		 *
-		 * The list of free space joins adjacent free blocks,
-		 * ignoring device boundries.  If we want to keep track
-		 * of this information per device, we'll just have to
-		 * extract it ourselves.
-		 */
-
-		while (top / dmmax != bottom / dmmax)
-			{
-			next_block = ((bottom + dmmax) / dmmax);
-			perdev[(bottom / dmmax) % nswdev] +=
-				next_block * dmmax - bottom;
-			bottom = next_block * dmmax;
-			}
-		perdev[(bottom / dmmax) % nswdev] +=
-			top - bottom + 1;
-
-#if  __FreeBSD_version >= 220000
-		swapptr = head.rl_next;
-#else
-		swaplist = head.rl_next;
-#endif
-		}
-
-	header = getbsize(&hlen, &blocksize);
-	div = blocksize / 512;
-	avail = npfree = 0;
-	for (i = 0; i < nswdev; i++)
-		{
-		int xsize, xfree;
-
-		/*
-		 * Don't report statistics for partitions which have not
-		 * yet been activated via swapon(8).
-		 */
-
-		xsize = sw[i].sw_nblks;
-		xfree = perdev[i];
-		used = xsize - xfree;
-		npfree++;
-		avail += xsize;
-		}
-
-	/* 
-	 * If only one partition has been set up via swapon(8), we don't
-	 * need to bother with totals.
-	 */
-	*retavail = avail << 9;
-	*retfree = nfree << 9;
-	used = avail - nfree;
-	free(sw); free(perdev);
-#endif /* __FreeBSD_version >= 400000 */
 
 	/* increase counter, no errors occurs */
-	warning++; 
+	warning++;
 
 	return  (int)(((double)used / (double)avail * 100.0) + 0.5);
 	}
@@ -990,13 +574,6 @@ swapmode(unsigned long long *retavail, unsigned long long *retfree)
 static int
 get_bufspace(guint64 *bufspacep)
 	{
-#if  __FreeBSD_version < 300000
-	u_int	bufspace;
-
-	if (kvm_read(kvmd, nl_mem[N_BUFSPACE].n_value, (char *)&bufspace,
-		     sizeof(bufspace)) != sizeof(bufspace))
-		return 0;
-#else
 	static char	*name = "vfs.bufspace";
 	static int	oid_bufspace[CTL_MAXNAME + 2];
 	static size_t	oid_bufspace_len = sizeof(oid_bufspace);
@@ -1015,12 +592,11 @@ get_bufspace(guint64 *bufspacep)
 	if (sysctl(oid_bufspace, oid_bufspace_len,
 		   &bufspace, &bufspace_len, NULL, 0) < 0)
 		return 0;
-#endif
+
 	*bufspacep = bufspace;
-	return 1;	
+	return 1;
 	}
 
-#if __FreeBSD_version >= 410000
 struct mibtab {
     char	*name;
     int		oid[CTL_MAXNAME + 2];
@@ -1050,7 +626,6 @@ static struct mibtab mibs[] = {
 };
 
 #define	PROC_MEMINFO_FILE	"/compat/linux/proc/meminfo"
-#endif
 
 #ifndef VM_TOTAL
 #define VM_TOTAL	VM_METER
@@ -1073,21 +648,7 @@ gkrellm_sys_mem_read_data(void)
 	struct vmtotal	vmt;
 	size_t		length_vmt = sizeof(vmt);
 	static int	oid_vmt[] = { CTL_VM, VM_TOTAL };
-#if __FreeBSD_version >= 410000
 	gint		i;
-#else
-	guint64		x_used;
-	struct vmmeter	sum;
-#endif
-
-#if 0
-	/* mem.c does a force_meminfo_update() before calling this */
-	/* Collecting meminfo data is expensive under FreeBSD, so
-	|  take extra precautions to minimize reading it.
-	*/
-	if (!GK.ten_second_tick && !force_meminfo_update())
-		return;
-#endif
 
 	if (pshift == 0)
 		{
@@ -1096,7 +657,6 @@ gkrellm_sys_mem_read_data(void)
 		}
 
 	shared = 0;
-#if __FreeBSD_version >= 410000
 	if (!first_time_done)
 		{
 		for (i = 0; mibs[i].name; ++i)
@@ -1134,39 +694,6 @@ gkrellm_sys_mem_read_data(void)
 		}
 	swappgsin = mibs[MIB_V_SWAPPGSIN].value;
 	swappgsout = mibs[MIB_V_SWAPPGSOUT].value;
-#else
-	if (kvmd == NULL)
-		return;
-	if (nl_mem[0].n_type == 0)
-		if (kvm_nlist(kvmd, nl_mem) < 0 || nl_mem[0].n_type == 0)
-			return;
-	if (kvm_read(kvmd, nl_mem[N_CNT].n_value, (char *)&sum,
-		     sizeof(sum)) != sizeof(sum))
-		return;
-
-	total = (sum.v_page_count - sum.v_wire_count) << pshift;
-	x_used = (sum.v_active_count + sum.v_inactive_count) << pshift;
-	free = sum.v_free_count << pshift;
-	if (sysctl(oid_vmt, 2, &vmt, &length_vmt, NULL, 0) == 0)
-		shared = vmt.t_rmshr << pshift;
-	get_bufspace(&buffers);
-	cached = sum.v_cache_count << pshift;
-	used = x_used - buffers - cached;
-	gkrellm_mem_assign_data(total, used, free, shared, buffers, cached);
-
-	if (swappgsin < 0)
-		{
-		dpagein = 0;
-		dpageout = 0;
-		}
-	else
-		{
-		dpagein = (sum.v_swappgsin - swappgsin) << pshift;
-		dpageout = (sum.v_swappgsout - swappgsout) << pshift;
-		}
-	swappgsin = sum.v_swappgsin;
-	swappgsout = sum.v_swappgsout;
-#endif
 
 	if (dpagein > 0 || dpageout > 0 || first_time_done == 0)
 		{
@@ -1396,11 +923,7 @@ static VoltDefault	voltdefault0[] =
 
 #include <dirent.h>
 #include <machine/cpufunc.h>
-#if __FreeBSD_version >= 500042
 #include <dev/smbus/smb.h>
-#elif __FreeBSD_version >= 300000
-#include <machine/smb.h>
-#endif
 
 /* Interface types */
 #define INTERFACE_IO		0
@@ -1435,13 +958,14 @@ get_data(int iodev, u_char command, int interface, u_char *ret)
 		outb(WBIO1, command);
 		byte = inb(WBIO2);
 		}
-#if __FreeBSD_version >= 300000
 	else if (interface == INTERFACE_SMB)
 		{
 		struct smbcmd cmd;
 
 		bzero(&cmd, sizeof(cmd));
+#if __FreeBSD_version < 1100070
 		cmd.data.byte_ptr = (char *)&byte;
+#endif
 		cmd.slave         = 0x5a;
 		cmd.cmd           = command;
 		if (ioctl(iodev, SMB_READB, (caddr_t)&cmd) == -1)
@@ -1449,8 +973,10 @@ get_data(int iodev, u_char command, int interface, u_char *ret)
 			close(iodev);
 			return FALSE;
 			}
-		}
+#if __FreeBSD_version >= 1100070
+		byte = (u_char)cmd.rdata.byte;
 #endif
+		}
 	else
 		{
 		return FALSE;
@@ -1541,6 +1067,13 @@ gkrellm_sys_sensors_get_voltage(gchar *path, gint id,
 	return FALSE;
 	}
 
+gboolean
+gkrellm_sys_sensors_get_power(gchar *path, gint id,
+		gint iodev, gint interface, gfloat *power)
+	{
+	return FALSE;
+	}
+
 struct freebsd_sensor {
 	gint type;
 	gchar *id_name;
@@ -1564,7 +1097,7 @@ gkrellm_sys_sensors_init(void)
 	GList		*list;
 	struct freebsd_sensor *sensor;
 
-	/* Do intial daemon reads to get sensors loaded into sensors.c
+	/* Do initial daemon reads to get sensors loaded into sensors.c
 	*/
 	gkrellm_sys_sensors_mbmon_check(TRUE);
 
@@ -1652,17 +1185,14 @@ static void
 scan_for_sensors(void)
 	{
 	gchar		*chip_dir = SENSORS_DIR;
-#if __FreeBSD_version >= 300000
 	DIR		*dir;
 	struct dirent	*dentry;
-#endif
 	gchar		temp_file[256], id_name[8];
 	gint		iodev = -1, interface = -1, id;
 	gint		fandiv[3];
 	u_char		byte;
 	gboolean	found_sensors = FALSE;
 
-#if __FreeBSD_version >= 300000
 	if ((dir = opendir(chip_dir)) != NULL)
 		{
 		while ((dentry = readdir(dir)) != NULL)
@@ -1690,7 +1220,6 @@ scan_for_sensors(void)
 			}
 		closedir(dir);
 		}
-#endif
 	if (!found_sensors)
 		{
 		snprintf(temp_file, sizeof(temp_file), "%s/%s",
@@ -1765,6 +1294,13 @@ gkrellm_sys_sensors_get_voltage(gchar *path, gint id,
 	return FALSE;
 	}
 
+gboolean
+gkrellm_sys_sensors_get_power(gchar *path, gint id,
+		gint iodev, gint interface, gfloat *power)
+	{
+	return FALSE;
+	}
+
 gboolean
 gkrellm_sys_sensors_init(void)
 	{
