@@ -1,7 +1,7 @@
 /* GKrellM
 |  Copyright (C) 1999-2021 Bill Wilson
 |
-|  Authors:  Bill Wilson    billw@gkrellm.net
+|  Authors:  Bill Wilson
 |            Stefan Gehn    stefan+gkrellm@srcbox.net
 |  Latest versions might be found at:  https://gkrellm.srcbox.net
 |
