@@ -1,8 +1,10 @@
 /* GKrellM
-|  Copyright (C) 1999-2008 Bill Wilson
+|  Copyright (C) 1999-2021 Bill Wilson
+|                2007-2024 Stefan Gehn
 |
-|  Author:  Bill Wilson    billw@gkrellm.net
-|  Latest versions might be found at:  http://gkrellm.net
+|  Authors:  Bill Wilson
+|            Stefan Gehn    stefan+gkrellm@srcbox.net
+|  Latest versions might be found at:  https://gkrellm.srcbox.net
 |
 |
 |  GKrellM is free software: you can redistribute it and/or modify it
@@ -47,14 +49,11 @@ gkrellm_winop_reset(void)
 void
 gkrellm_winop_options(gint argc, gchar **argv)
 	{
-	HWND hWnd = GDK_WINDOW_HWND(gkrellm_get_top_window()->window);
+	// This essentially hides the taskbar entry on win32, unfortunately
+	// gtk_window_set_skip_taskbar_hint() does not have any effect
+	gtk_window_set_type_hint(GTK_WINDOW(gkrellm_get_top_window()), GDK_WINDOW_TYPE_HINT_UTILITY);
 
-    // Get rid of task list icon
-	SetWindowLong(hWnd, GWL_EXSTYLE, WS_EX_TOOLWINDOW);
-
-	// Set stay-on-top flag if requested
-	if (_GK.on_top)
-		SetWindowPos(hWnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
+	gkrellm_winop_state_above(_GK.on_top);
 	}
 
 void
@@ -131,6 +130,7 @@ gkrellm_winop_place_gkrellm(gchar *geom)
 		newY = atoi(part);
 		if (ysign == -1)
 			newY = _GK.h_display - h_gkrell + newY;
+		free(part);
 		}
 
 	if (   newX >= 0 && newX < _GK.w_display - 10
@@ -340,6 +340,8 @@ gkrellm_winop_state_skip_pager(gboolean state)
 void
 gkrellm_winop_state_above(gboolean state)
 	{
+	GtkWindow *window = GTK_WINDOW(gkrellm_get_top_window());
+	gtk_window_set_keep_above(window, state);
 	}
 
 void
